uniform extern texture g_samSrcColor;
uniform extern texture g_bumpMap;
uniform extern texture g_paperTex;
uniform extern texture g_refractTex;
uniform extern texture stencil;
uniform extern texture reflection;
uniform extern texture player;

extern bool swim;

float Timer : TIME;

extern float xcam;
extern float ycam;

sampler textureSamplers[7] =
{
	sampler_state
	{
		Texture = <g_bumpMap>;
	},

	sampler_state
	{
		Texture = <g_samSrcColor>;
	},
	
	sampler_state
	{
		Texture = <stencil>;
	},
	
	sampler_state
	{
		Texture = <reflection>;
	},
	
	sampler_state
	{
		Texture = <player>;
	},
	
	sampler_state
	{
		Texture = <g_paperTex>;
	},
	
	sampler_state
	{
		Texture = <g_refractTex>;
	},
	
};


//noise effect intensity value (0 = no effect, 1 = full effect)
float fNintensity = 0.4;
//scanlines effect intensity value (0 = no effect, 1 = full effect)
float fSintensity = 0.9;
//scanlines effect count value (0 = no effect, 4096 = full effect)
float fScount = 600;

#define RGB

float4 Scanlines( float2 Tex : TEXCOORD0 ) : COLOR0
{
   /*float2 t = Tex;		
	float x=t.x*t.y*123456 * Timer;
	x=fmod(x,13) * fmod(x,123);	
	float dx = fmod(x,0.01);
	float dy = fmod(x,0.012);
	float4 c=tex2D(textureSamplers[1], t+float2(dx,dy));		
	return c;*/
	
	// sample the source
    float4 cTextureScreen = tex2D(textureSamplers[1], Tex.xy);

    // make some noise
    float x = Tex.x * Tex.y * Timer * 100000;
    x = fmod(x, 13) * fmod(x, 123);   
    float dx = fmod(x, 0.01);
   
    // add noise
    float3 cResult = cTextureScreen.rgb + cTextureScreen.rgb * saturate(0.1f + dx.xxx * 100);

    // get us a sine and cosine
    float2 sc; sincos(Tex.y * fScount, sc.x, sc.y);

    // add scanlines
    cResult += cTextureScreen.rgb * float3(sc.x, sc.y, sc.x) * fSintensity;
   
    // interpolate between source and result by intensity
    //cResult = lerp(cTextureScreen, cResult, saturate(fNintensity));

    // return with source alpha
    float4 result;// = float4(cResult, cTextureScreen.a);
    result.rgb = cResult;
    result.a = cTextureScreen.a;
    
    return result;

}

float4 FactoryShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig;
   float2 rel = Tex;
   
   orig = tex2D(textureSamplers[1], Tex);

   float4 blur = tex2D(textureSamplers[1],Tex);
   float4 temp;
      
   float xstep = 1.0/800;
   float ystep = 1.0/600;
   
   //blur.rgb = 0;
   //blur.a = 1;
   
   Tex.x -= xstep;
   Tex.y -= ystep;
   for(int i=0;i<3;++i)
   {
	  for(int j=0; j<3;++j)
	  {
		 blur += tex2D(textureSamplers[1], Tex);
		 
		 Tex.y += ystep;
		 if(rel.y > 1.0 - 6*ystep)
			Tex.y = 1.0 - 6*ystep;
		 else if(rel.y < 5*ystep)
			Tex.y = 5*ystep;
	  }	  
	  Tex.x += xstep;
	  if(rel.x > 1.0)
			Tex.x = 1.0;
	  else if(rel.x < 0.0)
			Tex.x = 0.0;
			
   }
   
   blur /= 7.0;
   
   blur.rgb = (blur.r * 0.2 + blur.g *0.4 + blur.b*0.4);
   
   
   float4 bw;
   
   bw.rgb = (orig.r * 0.2 + orig.g *0.4 + orig.b*0.4);
   bw.a = orig.a;

   orig.gb *= 1.3;
   orig.r *= 1;
   
   orig = orig*0.5 + bw*0.5;
   
   orig*=1.1;
   
   return orig;//*0.6 + blur*0.4;
}

float4 ConstructionShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig;
   float2 center = {0.5,0.5};
   
   float dist = distance(Tex,center)*0.7;
   if(dist > 1)
      dist = 1;
   dist = 1-dist;
   
   orig = tex2D(textureSamplers[1], Tex);
 
   orig.r *= 1.5;
   orig.gb *= 1.3;
   return orig;
}

float4 MountainShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig, ori;
   float2 center = {0.5,0.5};
   
   float dist = distance(Tex,center)*0.7;
   if(dist > 1)
      dist = 1;
   dist = 1-dist;
   
   ori = tex2D(textureSamplers[1], Tex);
   orig = tex2D(textureSamplers[1], Tex);
 
   orig.g *= 0.8;
   orig.b *= 0.7;
   
   orig.rgb = sqrt(orig.rgb * ori.rgb)+0.1;
   
   return orig;
}

float4 NullShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig = tex2D(textureSamplers[1], Tex);
   
   float2 center = {0.5,0.5};

   
   float3 normal = normalize(orig.rgb);
   float3 normal2 = normalize(orig.rbg);
   float3 light;
   light.x = Tex.x;
   light.y = Tex.y;
   
   //light.xy = -light.xy;
   
   light.z = 1.0;
   normalize(light);
   
   float3 light2 = light;
   
   light2.z *= -1;

   float3 view;
   view.xy = Tex.xy;
   view.z = 1.0;
   
   float3 halfway = normalize(view + light);
   
   float3 white = {0.7,0.7,0.7};
   float3 white2 = {0.8,0.8,0.8};
   
   float3 diffuse = saturate(dot(normal, light)) * white;
   
   float3 specular = pow(saturate(dot(normal, halfway)), 256) * white2;
   
   specular += pow(saturate(dot(normal, halfway)), 16) * white;

   specular.rgb = clamp(specular.rgb,0.0,1.0);
   
   specular = floor((specular)*5.0)/5.0;
   
   orig.rgb = specular;
   orig.a = 0.2;
   
   return orig;
}

float4 EVILFactoryBGShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig = tex2D(textureSamplers[1], Tex);
   
   float2 center = {0.5,0.5};
   
   float2 offset;
   
   offset.x = xcam;
   offset.y = ycam;
		
   offset += Tex;
		
	if(offset.x > 1.0)
		offset.x -= 1.0;
	else if(offset.x < 0.0)
		offset.x += 1.0;
		
	if(offset.y > 1.0)
		offset.y -= 1.0;
	else if(offset.y < 0.0)
		offset.y += 1.0;

   
   float3 normal = normalize(orig.rgb);
   float3 normal2 = normalize(orig.rbg);
   float3 light, light2;
   
   light.x = 0.0;
   light.y = Tex.y + 1.3;
   
   light2.x = 1.0;
   light2.y = Tex.y + 1.3;
   light2.z = 1.0;
   
   
   light.z = 1.0;
   normalize(light);
   
   float3 view;
   view.xy = Tex.xy;
   view.z = 1.0;
   
   float3 halfway = normalize(view + light);
   float3 halfway2 = normalize(view + light2);
   
   float3 white = {0.7,0.0,0.0};
   float3 white2 = {1.0,0.8,0.9};
   
   //float3 diffuse = saturate(dot(normal, light)) * white;
   
   float3 specular = pow(saturate(dot(normal, halfway)), 128) * white2;
   
   specular += pow(saturate(dot(normal, halfway2)), 128) * white2;
   
   specular += pow(saturate(dot(normal, halfway)), 4) * white;

   specular.rgb = clamp(specular.rgb,0.0,1.0);
   
   //specular = floor((specular)*50.0)/50.0;
   
   orig.rgb = specular;
   orig.a = 0.6;
   
   return orig;
}

float4 FactoryBGShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig = tex2D(textureSamplers[1], Tex);
   
   float2 center = {0.5,0.5};

   
   float3 normal = normalize(orig.rgb);
   float3 normal2 = normalize(orig.rbg);
   float3 light;
   light.x = Tex.x;
   light.y = Tex.y+1.2;
   
   //light.xy = -light.xy;
   
   light.z = 1.0;
   normalize(light);
   
   float3 light2 = light;
   
   light2.z *= -1;

   float3 view;
   view.xy = Tex.xy;
   view.z = 1.0;
   
   float3 halfway = normalize(view + light);
   
   float3 white = {0.6,0.6,0.9};
   float3 white2 = {0.8,0.8,0.8};
   
   float3 diffuse = saturate(dot(normal, light)) * white;
   
   float3 specular = pow(saturate(dot(normal, halfway)), 256) * white2;
   
   specular += pow(saturate(dot(normal, halfway)), 16) * white;

   specular.rgb = clamp(specular.rgb,0.0,1.0);
   
   //specular = floor((specular)*5.0)/5.0;
   
   orig.rgb = specular;
   orig.a = 0.5;
   
   return orig;
}


float4 SubconShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
   float4 orig;
   float2 center = {0.5,0.5};
   float2 offset;
   
   offset.x = xcam;
   offset.y = ycam;
		
   offset += Tex;
		
	if(offset.x > 1.0)
		offset.x -= 1.0;
	else if(offset.x < 0.0)
		offset.x += 1.0;
		
	if(offset.y > 1.0)
		offset.y -= 1.0;
	else if(offset.y < 0.0)
		offset.y += 1.0;
   
   float4 paper = tex2D(textureSamplers[5], offset.xy );
   
   
   
   float2 origtex = Tex;
   
   Tex.x += 0.03*(paper.r-1);
   Tex.y += 0.03*(paper.r-1);
   
   float4 stenciltex = tex2D(textureSamplers[2], Tex );
   
   orig = tex2D(textureSamplers[1], Tex);
 
   orig.rgb = orig.r*0.3 + orig.g*0.3 + orig.b * 0.4;
   orig.rgb = 1-orig.rgb;
   //orig.rg *= 0.5;
   orig.b = 1.0;
   
   if(stenciltex.b > 0)
   {
		//orig = tex2D(textureSamplers[1], Tex );
		//orig.rg += 0.4;
		orig.rg = 0.8 - orig.rg;
   }
   
   orig.rgb = sqrt(orig.rgb*paper.rgb*paper.rgb*paper.rgb*paper.rgb*paper.rgb);
   //orig.rgb = sqrt(orig.rgb*paper.rgb*paper.rgb*paper.rgb*paper.rgb*paper.rgb);
     
   return orig;
}

float4 IceShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 result;
	float4 bump;
	float4 bump2;
	float4 stenciltex;
	float2 offset;
	float2 offset2;
	float2 original = Tex;
	
	float2 center = {0.5,0.5};
	float dist = distance(Tex,center);
	if(dist > 1)
	  dist = 1;
	
	stenciltex = tex2D(textureSamplers[2], Tex );
	
	
		offset.x = xcam;
		offset.y = ycam;
		
		offset += Tex;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		offset2 = offset;
		
		/*offset.x += Timer;
		offset.y -= Timer;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;*/
			
		
		bump = tex2D(textureSamplers[0], offset.xy );
		bump2 = tex2D(textureSamplers[0],offset2.xy);
		
		//WATER DISABLED
		Tex.x = Tex.x + (bump.r-0.5)*0.1*(dist*2);
		Tex.y = Tex.y + (bump.r-0.5)*0.1*(dist*2);
     
     
	
	if(stenciltex.r > 0.95)
	{
		result = tex2D(textureSamplers[1], Tex );
		result.rg *= 0.6;
	}
	else
	{
		result = tex2D(textureSamplers[1], original );
	}
	
	float4 reflectionstencil = tex2D(textureSamplers[3],original);
	
	original.y = 1.15-original.y;
	
	Tex.x = original.x + (bump.r-0.5)*0.1*(2);
	Tex.y = original.y + (bump.r-0.5)*0.1*(2);
	float4 playerdraw = tex2D(textureSamplers[4], Tex);
	
	if(reflectionstencil.r == 0 && reflectionstencil.g == 0 && reflectionstencil.b == 0)
	{
		playerdraw.rgba = 0;
	}
	//if(playerdraw.r > 0 || playerdraw.g > 0 || playerdraw.b > 0)
	if(playerdraw.a > 0)
	{
		playerdraw.rgb *= 0.5;
		result.rgb *= 0.7;
		result.rgb += playerdraw.rgb;
		//return sqrt(result*playerdraw);
	}	
	
	
	
   
	//result.r *= 0.8;
	result.b *= 1.2;
	
	result.rgb -= (dist)/8;
	return result;
}

float PixelKernel[9] =
{
    -4,
    -3,
    -2,
    -1,
     0,
     1,
     2,
     3,
     4,
};

static const float BlurWeights[9] = 
{
    0.026995,
    0.064759,
    0.120985,
    0.176033,
    0.199471,
    0.176033,
    0.120985,
    0.064759,
    0.026995,
};


float4 WaterShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 result;
	float4 bump;
	float4 stenciltex, stenciltexxm;
	float2 offset;
	float2 original = Tex;
	float4 refract;
	float4 refract2;
	
	float xstep = 1.0/800;
    float ystep = 1.0/600;
   
	stenciltex = tex2D(textureSamplers[2],Tex);
	
	float2 center;
		center.x = 0.5;
		center.y = 0.5;
		
		//float dist = 0.5-distance(Tex,center);
		//if(dist < 0)
		//	dist = 0;
		float dist = 0.5;
			
	if(true)
	{
		offset.x = xcam;
		offset.y = ycam;
		
		offset += Tex;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		offset.x += Timer;
		offset.y -= Timer;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		
		bump = tex2D(textureSamplers[0], offset.xy );
		
		Tex.x = Tex.x + (bump.r-0.5)*0.1*(dist);
		Tex.y = Tex.y + (bump.r-0.5)*0.1*(dist);
		
		Tex.x = clamp(Tex.x,0.0,1.0);
		Tex.y = clamp(Tex.y,0.0,1.0);
		//Tex.x = Tex.x + (bump.r-0.5)*0.05;
		//Tex.y = Tex.y + (bump.r-0.5)*0.05;

	}
	
	if(stenciltex.r > 0.9)
	{
		result = tex2D(textureSamplers[1], Tex );
		result.rg *= 0.6;
	}
	else
	{
		result = tex2D(textureSamplers[1], original );	
	}
	
	refract = tex2D(textureSamplers[6],offset.xy);
	offset.x -= 2*Timer;
	refract2 = tex2D(textureSamplers[6],1.0-offset.xy);
	
	//result.a = 1-dist+.1;
	
	//return result;
	refract.rgb = sqrt(refract.rgb*refract.rgb*refract2.rgb*refract2.rgb);
	refract.rgb *= 3.0;
	refract.rgb *= 0.8 - Tex.y;
	result.rgb = result.rgb + refract.rgb*0.2;
	return result;
}

float4 TestShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 result;
	float4 bump;
	float4 stenciltex;
	float2 offset;
	float2 original = Tex;
	
	stenciltex = tex2D(textureSamplers[2], Tex );
	
	float2 center;
		center.x = 0.5;
		center.y = 0.5;
		
		float dist = 0.5-distance(Tex,center);
		if(dist < 0)
			dist = 0;
			
	if(true)
	{
		offset.x = xcam;
		offset.y = ycam;
		
		offset += Tex;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		offset.x += Timer;
		offset.y -= Timer;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		
		bump = tex2D(textureSamplers[0], offset.xy );
		
		Tex.x = Tex.x + (bump.r-0.5)*0.1*(dist*2);
		Tex.y = Tex.y + (bump.r-0.5)*0.1*(dist*2);

	}
	
	if(stenciltex.r > 0.9)
	{
		result = tex2D(textureSamplers[1], Tex );
		result.rg *= 0.6;
	}
	else
	{
		result = tex2D(textureSamplers[1], original );
	}
	
	
	//result.a = 1-dist+.1;
	
	return result;
}

float4 BeachShader ( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 result;
	float4 bump;
	float4 stenciltex;
	float2 offset;
	float2 original = Tex;
	
	stenciltex = tex2D(textureSamplers[2], Tex );
	
	float2 center;
		center.x = 0.5;
		center.y = 0.5;
		
		float dist = 0.5-distance(Tex,center);
		if(dist < 0)
			dist = 0;
			
	if(true)
	{
		offset.x = xcam;
		offset.y = ycam;
		
		offset += Tex;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		offset.x += Timer;
		offset.y -= Timer;
		
		if(offset.x > 1.0)
			offset.x -= 1.0;
		else if(offset.x < 0.0)
			offset.x += 1.0;
			
		if(offset.y > 1.0)
			offset.y -= 1.0;
		else if(offset.y < 0.0)
			offset.y += 1.0;
			
		
		bump = tex2D(textureSamplers[0], offset.xy );
		
		Tex.x = Tex.x + (bump.r-0.5)*0.1*(dist*2);
		Tex.y = Tex.y + (bump.r-0.5)*0.1*(dist*2);

	}
	
	if(stenciltex.b > 0.9)
	{
		result = tex2D(textureSamplers[1], Tex );
		result.rg *= 0.6;
	}
	else
	{
		result = tex2D(textureSamplers[1], original );
	}
	
	
	//result.a = 1-dist+.1;
	
	return result;
}

/*
float4 MyShader ( float2 Tex : TEXCOORD0, float2 Tex2) : COLOR0
{
    float4 Color;
    float4 Color2;
    float4 Color3;
    
    Tex.x = Tex.x + (sin((Tex.y+Timer/100)*25)*0.01);
    Tex.y = Tex.y + (sin((Tex.y+Timer/100)*10)*0.02);
    Color2 = tex2D( g_samSrcColor, Tex.xy);
    
    
    float xtemp = Tex.x;
    float ytemp = Tex.y;
    
    Tex.x = Tex.x + (sin((Tex.y+Timer/100)*50)*0.01);
    
    Tex.y += (sin((Tex.y+Timer/100)*100)*0.02);
    //Tex.x += Color2.rb;
    //Tex.y += Color2.rb;
    
    Color = tex2D( g_samSrcColor, Tex.xy);
    
    
    
    Color2.rgb *= .7;
    Color2.rg *= .8;
    
    //Color.rgb = Color2.rgb;
    
    Color.rgb *= .3;
    Color.rg *= .3;
    
    return Color + Color2;// + Color2*Timer*100;// + Color3;

*/

/*
// sample the source
	float4 cTextureScreen = tex2D( g_samSrcColor, Tex.xy);
	
	float distx = Tex.x;
	float disty = Tex.y;
	
	if (Tex.x > 0.5) distx = 1 - distx;
	if (Tex.y > 0.5) disty = 1 - disty;
	
	disty *= 4.0/3.0;
	
	cTextureScreen.rgb *= sqrt(distx*distx+disty*disty);
	
	//if(Tex.y < 0.5) cTextureScreen.rgb *= (Tex.y)*2;
	//if(Tex.y > 0.5) cTextureScreen.rgb *= (1-(Tex.y))*2;
	
	//if(Tex.x < 0.5) cTextureScreen.rgb *= (Tex.x)*2;
	//if(Tex.x > 0.5) cTextureScreen.rgb *= (1-(Tex.x))*2;

	// make some noise
	float x = Tex.x * Tex.y * Timer *  1000;
	x = fmod(x, 13) * fmod(x, 123);	
	float dx = fmod(x, 0.01);
	
	// add noise
	float3 cResult = cTextureScreen.rgb + cTextureScreen.rgb * saturate(0.1f + dx.xxx * 100);

	// get us a sine and cosine
	float2 sc; sincos(Tex.y * fScount, sc.x, sc.y);

	// add scanlines
	cResult += cTextureScreen.rgb * float3(sc.x, sc.y, sc.x) * fSintensity;
	
	// interpolate between source and result by intensity
	cResult = lerp(cTextureScreen, cResult, saturate(fNintensity));

	// convert to grayscale if desired
	#ifdef MONOCHROME
		cResult.rgb = (cResult.r * 0.3f + cResult.g * 0.59f + cResult.b * 0.11f);
	#endif

	// return with source alpha
	return float4(cResult, cTextureScreen.a);*/
//}

technique PostProcess
{
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 TestShader();
    }
}

technique FactoryPostProcess
{
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 FactoryShader();
    }
}

technique ConstructionPostProcess
{
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 ConstructionShader();
    }
}

technique IcePostProcess
{
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 IceShader();
    }
}

technique SubconPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 SubconShader();
	}
}

technique WaterPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 WaterShader();
	}
}

technique MountainPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 MountainShader();
	}
}

technique ScanlinePostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 Scanlines();
	}
}

technique BeachPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 BeachShader();
	}
}

technique NullPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 NullShader();
	}
}

technique FactoryBGPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 FactoryBGShader();
	}
}

technique EVILFactoryBGPostProcess
{
	pass p1
	{
		VertexShader = null;
		PixelShader = compile ps_2_0 EVILFactoryBGShader();
	}
}